@@ -14,7 +14,7 @@ struct _BlobSpan {
 struct _Blob {
   int y;
   int height;
-  BlobSpan data[0];
+  BlobSpan data[0];		/* slightly in violation of ANSI-C? */
 };
 
 
