@@ -33,8 +33,9 @@ int image_width;
 int image_height;
 
 /* These may be 32, 16, or 8 bpp pixels, depending on bpp. */
-int32 gray_pixels[256];
-int32 red_pixel;
+gulong gray_pixels[256];
+gulong red_pixel;
+gulong maroon_pixel;
 
 /* The region (in screen coordinates) that needs to be rendered from the
    bitmap to the screen image.
@@ -61,9 +62,15 @@ point cursor_pos, cursor_pos2;
 int cursor_icon, cursor_icon2;
 
 void create_ximage (GtkWidget *area, int width, int height) {
-  if (img != NULL) {
-    gdk_image_destroy(img);
-  }
+
+  if (img != NULL)
+    {
+      if ((image_width == width) && (image_height == height))
+	return;
+      else
+	gdk_image_destroy(img);
+    }
+
   image_width = width;
   image_height = height;
 
@@ -84,6 +91,9 @@ void create_ximage (GtkWidget *area, int width, int height) {
   case 2:
     memset (img->mem, 255, img->bpl * image_height);
     break;
+  case 3:
+    memset (img->mem, 255, img->bpl * image_height);
+    break;
   case 4:
     {
       int i;
@@ -207,6 +217,13 @@ static void gray_run (int y, int x_min, int x_max)
     for (i = x_min; i < x_max; i++)
       ((uint16 *)img_line)[i] = gray_pixels[127];
     break;
+  case 3:
+    for (i = x_min; i < x_max; i++) 
+      {
+	((uint8 *)img_line)[i * 3] = gray_pixels[127] >> 16;
+	((uint8 *)img_line)[i * 3 + 1] = (gray_pixels[127] >> 8) & 0xff;
+	((uint8 *)img_line)[i * 3 + 2] = gray_pixels[127] & 0xff;
+      }
   case 4:
     for (i = x_min; i < x_max; i++)
       ((uint32 *)img_line)[i] = gray_pixels[127];
@@ -337,6 +354,14 @@ void bitmap_render_to_ximage (int x, int y, int width, int height) {
       for (i = x; i < bm_x_max; i++)
 	((uint16 *)img_line)[i] = grays[(int)accum[i]];
       break;
+    case 3:
+      for (i = x; i < bm_x_max; i++) 
+	{
+	  ((uint8 *)img_line)[i * 3] = grays[(int)accum[i]] >> 16;
+	  ((uint8 *)img_line)[i * 3 + 1] = (grays[(int)accum[i]] >> 8) & 0xff;
+	  ((uint8 *)img_line)[i * 3 + 2] = grays[(int)accum[i]] & 0xff;
+	}
+      break;
     case 4:
       for (i = x; i < bm_x_max; i++)
 	((uint32 *)img_line)[i] = grays[(int)accum[i]];
@@ -389,6 +414,11 @@ void cursor_render (const rect *clip_rect) {
 	case 2:
 	  ((uint16 *)img_line)[k] = red_pixel;
 	  break;
+        case 3:
+          ((uint8 *)img_line)[k * 3] = red_pixel >> 16;
+          ((uint8 *)img_line)[k * 3 + 1] = (red_pixel >> 8) & 0xff;
+          ((uint8 *)img_line)[k * 3 + 2] = red_pixel & 0xff;
+          break;
 	case 4:
 	  ((uint32 *)img_line)[k] = red_pixel;
 	  break;
@@ -619,13 +649,17 @@ void draw_update (input_state *last, input_state *ins, ToolInfo *tool_info)
 
   blob_bounds (b_union, &rec);
   bitmap_update (&rec);
-  /*  undo_update (&rec1); */
   undo_update (&rec);
 
   bitmap_draw_blob (b_union, (tool_info->tool == PEN_TOOL) ? 0 : 1);
   free (b_union);
 }
 
+void draw_end (input_state *ins, ToolInfo *tool_info)
+{
+  undo_end ();
+}
+
 /* Draw a chunk of the draw_region. */
 int draw (GtkWidget *area) {
   rect rec;
